import db from "../../db"
import { SheetModel } from "../../db/sheet"
import { Context } from "../../types"
import llm from "../../utils/llm"
import { extractSheetIds, validateLink } from "../../utils/string"

/**
 * 爬取网页内容
 * @param {Context} ctx - 上下文对象
 * @param {string} link - 网页链接
 * @returns {Promise<any>} - 返回爬取结果
 * @throws {Error} - 当爬取失败时抛出错误
 */
export const crawlWebPage = async (
  ctx: Context,
  link: string
): Promise<any> => {
  const { attachService } = ctx
  const crawRes = await attachService.crawlWeb(link)
  if (!crawRes || crawRes?.code) throw new Error("网页抓取失败")
  return crawRes
}

/**
 * 生成网页简报
 * @param {Context} ctx - 上下文对象
 * @param {string} userDescription - 用户描述
 * @param {any} content - 网页内容
 * @returns {Promise<string>} - 返回简报内容
 * @throws {Error} - 当生成简报失败时抛出错误
 */
export const generateSummary = async (
  ctx: Context,
  userDescription: string,
  content: any
): Promise<string> => {
  const { requestId } = ctx
  const llmRes = (await llm.invoke(
    "summaryWeb",
    {
      description: userDescription,
      content: content,
    },
    requestId,
    1
  )) as string
  if (!llmRes) throw new Error("模型总结失败")
  return llmRes
}

/**
 * 获取简报表格
 * @param {Context} ctx - 上下文对象
 * @returns {Promise<{sheetToken: string, range: string, sheetUrl: string}> | null} - 返回表格信息或null
 */
const getReportSheet = async (ctx: Context) => {
  const chat = await db.chat.getAndCreate(ctx)
  if (!chat || !chat?.sheet || !chat?.expand?.sheet) {
    return null
  }
  const { sheetToken, range, sheetUrl } = chat.expand.sheet as SheetModel
  return { sheetToken, range, sheetUrl }
}

/**
 * 插入到表格
 * @param {Context} ctx - 上下文对象
 * @param {string} link - 网页链接
 * @param {string} userDescription - 用户描述
 * @param {string} llmRes - 简报内容
 * @param {string} sheetToken - 表格token
 * @param {string} range - 表格range
 * @returns {Promise<string>} - 返回表格链接
 */
const insert2Sheet = async (
  ctx: Context,
  link: string,
  userDescription: string,
  llmRes: string,
  sheetToken: string,
  range: string
) => {
  const {
    larkBody: { userId },
    logger,
    larkService,
  } = ctx
  try {
    await larkService.sheet.insertRows(sheetToken, range, [
      [
        userId || "",
        link,
        userDescription,
        llmRes,
        new Date().toLocaleString("zh-CN", { timeZone: "Asia/Shanghai" }),
      ],
    ])
  } catch (error: any) {
    logger.error(`Failed to insert2Sheet: ${error}`)
  }
}

/**
 * 生成简报卡片
 * @param {Context} ctx - 上下文对象
 * @param {string} link - 网页链接
 * @param {string} userDescription - 用户描述
 * @param {string} llmRes - 简报内容
 * @param {string} sheetLink - 表格链接
 * @returns {any} - 返回卡片对象
 */
const genReportCard = (
  ctx: Context,
  link: string,
  userDescription: string,
  llmRes: string,
  sheetLink: string
) => {
  const { larkCard } = ctx
  const cardGender = larkCard.child("webAgent")
  const description = userDescription
    ? `**用户描述📝**\n${userDescription}\n`
    : ""
  const sheetLinkMd = sheetLink
    ? `**小提示🔍**
您可以在[这里](${sheetLink})查看往期简报哦！`
    : ""
  const fullContent = `
感谢您使用小煎蛋简报🍳！以下是为您总结的简报内容：

**网站地址🌟**
[${link}](${link})

${description}

**AI简报🌈**
${llmRes}

${sheetLinkMd}
`
  const content = sheetLink ? fullContent : llmRes

  return cardGender.genCard("markdownSuccessCard", {
    content,
  })
}

/**
 * 处理mify爬虫错误
 * @param {string} llmRes - 爬虫结果
 * @returns {string} - 返回错误信息
 */
const manageMifyCrawlerErr = (llmRes: string) => {
  const errMap: Record<string, string> = {
    crawlerErr: "网页抓取失败",
    larkErr: "文档获取失败或者文档为空",
    authErr: "请确保小煎蛋有对文档的读取权限，可以私发小煎蛋设置权限",
  }
  if (!llmRes) return "模型总结失败"
  if (errMap[llmRes]) return errMap[llmRes]
  if (llmRes.includes("mify爬虫调用失败")) return llmRes
  return ""
}

/**
 * 生成简报
 * @param {Context} ctx - 上下文对象
 * @param {string} link - 网页链接
 * @param {string} userDescription - 用户描述
 */
const agent = async (ctx: Context, link: string, userDescription: string) => {
  const {
    larkService: { message },
    larkCard,
    logger,
    attachService,
  } = ctx
  const cardGender = larkCard.child("webAgent")
  try {
    // 校验链接是否合法
    validateLink(link)
    // 发送一个loading卡片
    await message.updateOrReply(
      cardGender.genPendingCard("正在为您收集简报，请稍等片刻~")
    )
    // // 抓取网页
    // const crawRes = await crawlWebPage(ctx, link)
    // // 调用模型生成简报
    // const llmRes = await generateSummary(ctx, userDescription, crawRes)

    // 获取要插入的表格信息
    const sheetInfo = await getReportSheet(ctx)

    // 调用mify服务生成简报
    const llmRes = await attachService.mifyCrawler(link, userDescription)

    // 处理mify爬虫错误
    const mifyErr = manageMifyCrawlerErr(llmRes)

    if (sheetInfo) {
      // 插入到表格
      await insert2Sheet(
        ctx,
        link,
        userDescription,
        mifyErr ? "暂无总结" : llmRes,
        sheetInfo.sheetToken,
        sheetInfo.range
      )
    } else if (mifyErr) {
      throw new Error(mifyErr)
    }

    // 发送简报卡片
    await message.updateOrReply(
      genReportCard(
        ctx,
        link,
        userDescription,
        llmRes,
        sheetInfo?.sheetUrl || ""
      )
    )
  } catch (error: any) {
    logger.error(`Failed gen report: ${error}`)
    await message.updateOrReply(
      cardGender.genErrorCard(`简报生成失败: ${error.message}`)
    )
  }
}

/**
 * 解析表格信息
 * @param {Context} ctx - 上下文对象
 * @param {string} sheetUrl - 表格链接
 * @returns {Promise<{sheetToken: string, range: string}>} - 返回表格信息
 * @throws {Error} - 当解析失败时抛出错误
 */
const parseSheetInfo = async (
  ctx: Context,
  sheetUrl: string
): Promise<{ sheetToken: string; range: string }> => {
  const { larkService } = ctx
  const token = extractSheetIds(sheetUrl)
  if (!token || !token.sheetToken) throw new Error("链接格式错误")

  const getRange = async (sheetToken: string) => {
    const res = await larkService.sheet.getMetaInfo(sheetToken)
    if (!res || !res?.data?.sheets?.length)
      throw new Error("获取工作表信息失败")
    return res.data.sheets[0].sheetId
  }
  // 如果不是Wiki链接，直接提取sheetToken和range
  if (!sheetUrl.includes("wiki")) {
    if (token.range) return token as { sheetToken: string; range: string }
    // 获取第一个工作表作为range
    token.range = await getRange(token.sheetToken)
    return token as { sheetToken: string; range: string }
  }
  // 如果是Wiki链接，需要先获取sheetToken
  const res = await larkService.wiki.getNodeInfo(token?.sheetToken)
  if (!res || !res?.data?.node?.obj_token) throw new Error("获取Wiki信息失败")
  token.sheetToken = res.data.node.obj_token
  if (token.range) return token as { sheetToken: string; range: string }
  token.range = await getRange(token.sheetToken)
  return token as { sheetToken: string; range: string }
}

/**
 * 设置简报表格
 * @param {Context} ctx - 上下文对象
 * @param {string} sheetUrl - 表格链接
 */
const setSheet = async (ctx: Context, sheetUrl: string) => {
  const { larkCard, larkService, logger } = ctx
  const cardGender = larkCard.child("webAgent")
  try {
    // 获取chat信息
    const chat = await db.chat.getAndCreate(ctx)
    if (!chat) throw new Error("获取聊天信息失败")
    // 获取是否已经有存在的sheet信息
    const existSheet = await db.sheet.getByUrl(sheetUrl)
    // 如果已经存在，则直接写入chat表
    if (existSheet) {
      const updateRes = await db.chat.update(chat.id, { sheet: existSheet.id })
      if (!updateRes) throw new Error("更新chat记录失败")
      await larkService.message.updateOrReply(
        cardGender.genSuccessCard("简报汇总表设置成功")
      )
      return
    }

    // 获取sheet的token和range
    const sheetInfo = await parseSheetInfo(ctx, sheetUrl)

    // 创建sheet记录
    const createRes = await db.sheet.create({
      sheetToken: sheetInfo.sheetToken,
      range: sheetInfo.range,
      sheetUrl,
    })
    if (!createRes) throw new Error("创建sheet记录失败")
    const updateRes = await db.chat.update(chat.id, { sheet: createRes.id })
    if (!updateRes) throw new Error("更新chat记录失败")
    await larkService.message.updateOrReply(
      cardGender.genSuccessCard("简报汇总表创建成功")
    )
  } catch (error: any) {
    logger.error(`Failed setSheet: ${error}`)
    await larkService.message.updateOrReply(
      cardGender.genErrorCard(`简报汇总表设置失败: ${error.message}`)
    )
  }
}

const reportAgent = {
  agent,
  setSheet, // 添加 setSheet 到导出的对象中
}

export default reportAgent
